# -*- coding: utf-8 -*-
from __future__ import unicode_literals
"""Contains the class magcalsys_solidstate_1D.

Used to compute 1-dimensional models for
magnetocaloric systems

"""

import heatcond
import time
import numpy as np


class magcalsys_solidstate_1D:

    """magcalsys_solidstate_1D class

    computes the thermal processes for 1-dimensional fully solid state
    magnetocaloric systems. The active magnetcic regenerative processes
    can be used with the several allowed demagnetization processes.
    Cascades of materials can also be simulated.

    """

    def __init__(self, fileName, ambTemperature=293,
                 leftThermalSwitch_length=10, rightThermalSwitch_length=10,
                 MCM_length=50, rightReservoir_length=15,
                 leftReservoir_length=15, MCM_material='Gd',
                 leftThermalSwitch_material='idealTS_hot',
                 rightThermalSwitch_material='idealTS_cold',
                 leftReservoir_material='Cu', rightReservoir_material='Cu',
                 freq=.1, dt=0.01, dx=0.002, stopCriteria=5e-8,
                 solverMode='implicit_k(x)', minCycleNumber=50,
                 maxCycleNumber=10000, demagnetizationSteps=1,
                 magnetizationSteps=1, demagnetizationMode='constant_right',
                 magnetizationMode='constant_left', cyclePoints=25,
                 boundaries=[0, 0], note=None, temperatureSensor='default',
                 heatPoints='default', mode='refrigerator', version=None,
                 restingTimeHot='default', restingTimeCold='default',
                 startingField='magnetization'):
        """loads the data and computes.

        fileName: file name where the temperature and heat flux are saved
        ambTemperature: ambient temperature of the whole system
        leftThermalSwitch_length: length of the left thermal switch
        rightThermalSwitch_length: length of the right thermal switch
        leftThermalSwitch_material: string for the material of the
            left thermal switch
        rightThermalSwitch_material: string for the material of the
            right thermal switch
        MCM_length: length of the magnetocaloric material
        MCM_material: string for the material of the magnetocaloric material
        leftReservoir_length: length of the left reservoir
        rightReservoir_length: length of the right reservoir
        leftReservoir_material: string for the material of the left reservoir
        rightReservoir_material: string for the material of the right reservoir
        freq: operating frequency
        dt: times step
        dx: space step
        stopCriteria: error threshold to stop the simulation
        solverMode: the solver
        minCycleNumber: minimum number of cycles that has to be computed
        maxCycleNumber: maximum number of cycles that has to be computed
        demagnetizationSteps: number of steps during the demagnetization
        magnetizationSteps: number of steps during the magnetization
        demagnetizationMode: mode of demagnetization
            modes can be constant_right, constant_left, accelerated_right,
            accelerated_left, decelerated_right, and decelerated_left
        magnetizationMode is the mode of magnetization
            modes can be constant_right, constant_left, accelerated_right,
            accelerated_left, decelerated_right, and decelerated_left
        cyclePoints: number of points recorded for each position for each cycle
        boundaries: list with the boundary conditions
        temperatureSensor: list of two space indexes used to determine the
            temperature span at the end of the simulation. The first term is
            the sensor at the hot end and the second at the cold end
        heatPoints: list of two space indexes used to determine the heat
            flux for the hot end (first term) and cold end (second term)
        mode: mode used for the power calculations (e.g. COP) performed at
            the end of the simulation
        version: heatrapy version (default is None)

        """

        # restingTimes definition
        if restingTimeHot == 'default':
            restingTimeHot = 0.

        if restingTimeCold == 'default':
            restingTimeCold = 0.

        # information for the log file
        print ''
        print ''
        print '######################################################'
        print ''
        print '------------------------------------------------------'
        print fileName
        print '------------------------------------------------------'
        print ''
        print 'heatconpy version:', version
        print 'Module: magcalsys_solidstate_1D'
        if note is not None:
            print ''
            print 'Note:', note
        print ''
        print 'Mode:', mode
        print 'System:', leftReservoir_material + '/' + \
            leftThermalSwitch_material + '/' + MCM_material + '/' + \
            rightThermalSwitch_material + '/' + rightReservoir_material
        print 'Dimensions (m):', str(dx * leftReservoir_length) + '/' + \
            str(dx * leftThermalSwitch_length) + '/' + str(dx * MCM_length) + \
            '/' + str(dx * rightThermalSwitch_length) + '/' + \
            str(dx * rightReservoir_length)
        print 'Number of points:', str(leftReservoir_length) + '/' + \
            str(leftThermalSwitch_length) + '/' + str(MCM_length) + '/' + \
            str(rightThermalSwitch_length) + '/' + str(rightReservoir_length)
        print 'dx (m):', dx
        print 'dt (s):', dt
        print 'Frequency (Hz):', freq
        print 'Hot resting time ratio:', restingTimeHot
        print 'Cold resting time ratio:', restingTimeCold
        print 'Solver:', solverMode
        print 'Magnetization mode:', magnetizationMode
        print 'Magnetization steps:', magnetizationSteps
        print 'Demagnetization mode:', demagnetizationMode
        print 'Demagnetization steps:', demagnetizationSteps
        print 'Starting Field:', startingField
        print 'Boundaries:', boundaries
        print 'Ambient temperature (K):', ambTemperature
        print 'Stop criteria:', stopCriteria
        print 'Time:', time.ctime()
        print ''

        # used to calculate the simulation time at the end
        startTime = time.time()

        if heatPoints == 'default':
            leftHeatSensor = leftReservoir_length - 3
            rightHeatSensor = -rightReservoir_length + 3
        else:
            leftHeatSensor = heatPoints[0]
            rightHeatSensor = heatPoints[1]

        if startingField != 'magnetization':
            initialState = True
        else:
            initialState = False

        # initializes the object for the simulation

        materials = [leftReservoir_material, leftThermalSwitch_material,
                     MCM_material, rightThermalSwitch_material,
                     rightReservoir_material]

        lRl = leftReservoir_length
        rRl = rightReservoir_length
        ltsl = leftThermalSwitch_length
        rtsl = rightThermalSwitch_length
        borders = [1, lRl + 1,
                   lRl + ltsl + 1,
                   lRl + ltsl + MCM_length + 1,
                   lRl + ltsl + MCM_length + rtsl + 1,
                   lRl + ltsl + MCM_length + rtsl + rRl + 1]
        heatPoints = [leftHeatSensor, rightHeatSensor]

        a = heatcond.heatcond_activemat_1D(ambTemperature, dx=dx, dt=dt,
                                           fileName=fileName,
                                           materials=materials,
                                           borders=borders,
                                           materialsOrder=[0, 1, 2, 3, 4],
                                           boundaries=boundaries,
                                           heatPoints=heatPoints,
                                           initialState=initialState)

        # defines some variable for the cycles
        if temperatureSensor == 'default':
            rightTemperatureSensor = -(rightReservoir_length / 2)
            leftTemperatureSensor = (leftReservoir_length / 2)
        else:
            rightTemperatureSensor = temperatureSensor[1]
            leftTemperatureSensor = temperatureSensor[0]

        value1 = ambTemperature
        value2 = ambTemperature
        i = 0
        period = (1. / freq)
        stopCriteria2 = 0.
        maximumPower = 0.
        maximumWorkingPower = 0.

        # cycle simulation
        while ((abs((value1 - value2) / value2) > stopCriteria or
                i < minCycleNumber or
                abs((value1 - value2) / value2) > stopCriteria2) and
               i < maxCycleNumber):

            stopCriteria2 = abs((value1 - value2) / value2)
            heatLeft = a.heatLeft
            heatRight = a.heatRight

            if startingField == 'magnetization':

                # MAGNETIZATION AND COMPUTATION

                # mode 1
                if magnetizationMode == 'constant_right':
                    for j in range(magnetizationSteps):
                        first = (leftReservoir_length + j *
                                 (leftThermalSwitch_length + MCM_length +
                                  rightThermalSwitch_length) /
                                 magnetizationSteps + 1)
                        second = (leftReservoir_length + (j + 1) *
                                  (leftThermalSwitch_length + MCM_length +
                                   rightThermalSwitch_length) /
                                  magnetizationSteps + 1)
                        a.activate(first, second)
                        time_interval = ((1 - restingTimeHot -
                                          restingTimeCold) *
                                         period / 2.)
                        time_interval = (time_interval /
                                         (magnetizationSteps))
                        a.compute(time_interval,
                                  int(1 / (freq * dt * cyclePoints)),
                                  solver=solverMode)

                # mode 2
                if magnetizationMode == 'accelerated_right':
                    for j in range(magnetizationSteps):
                        first = (leftReservoir_length + j *
                                 (leftThermalSwitch_length + MCM_length +
                                  rightThermalSwitch_length) /
                                 magnetizationSteps + 1)
                        second = (leftReservoir_length + (j + 1) *
                                  (leftThermalSwitch_length + MCM_length +
                                   rightThermalSwitch_length) /
                                  magnetizationSteps + 1)
                        a.activate(first, second)
                        totalmsteps = magnetizationSteps
                        delta_t = ((1 / (2 * (1. / (1. -
                                                    restingTimeHot -
                                                    restingTimeCold)) *
                                         freq *
                                         np.sqrt(totalmsteps))) *
                                   (np.sqrt(j + 1) -
                                    np.sqrt(j)))
                        write_interval = int(1 / (freq * dt *
                                                  cyclePoints))
                        a.compute(delta_t, write_interval,
                                  solver=solverMode)

                # mode 3
                if magnetizationMode == 'decelerated_right':
                    for j in range(magnetizationSteps):
                        first = (leftReservoir_length + j *
                                 (leftThermalSwitch_length + MCM_length +
                                  rightThermalSwitch_length) /
                                 magnetizationSteps + 1)
                        second = (leftReservoir_length + (j + 1) *
                                  (leftThermalSwitch_length + MCM_length +
                                   rightThermalSwitch_length) /
                                  magnetizationSteps + 1)
                        a.activate(first, second)
                        totalmsteps = magnetizationSteps
                        delta_t = ((1 / (2 * (1. / (1. -
                                                    restingTimeHot -
                                                    restingTimeCold)) *
                                         freq *
                                         np.sqrt(totalmsteps))) *
                                   (np.sqrt(magnetizationSteps - j) -
                                    np.sqrt(magnetizationSteps - j -
                                            1)))
                        write_interval = int(1 / (freq * dt *
                                                  cyclePoints))
                        a.compute(delta_t, write_interval,
                                  solver=solverMode)

                # mode 4
                if magnetizationMode == 'constant_left':
                    for j in range(magnetizationSteps - 1, -1, -1):
                        first = (leftReservoir_length + j *
                                 (leftThermalSwitch_length + MCM_length +
                                  rightThermalSwitch_length) /
                                 magnetizationSteps + 1)
                        second = (leftReservoir_length + (j + 1) *
                                  (leftThermalSwitch_length + MCM_length +
                                   rightThermalSwitch_length) /
                                  magnetizationSteps + 1)
                        a.activate(first, second)
                        time_interval = (((1 - restingTimeHot -
                                          restingTimeCold) * period /
                                         2.) / (magnetizationSteps))
                        write_interval = int(1 / (freq * dt *
                                                  cyclePoints))
                        a.compute(time_interval, write_interval,
                                  solver=solverMode)

                # mode 5
                if magnetizationMode == 'accelerated_left':
                    for j in range(magnetizationSteps - 1, -1, -1):
                        first = (leftReservoir_length + j *
                                 (leftThermalSwitch_length + MCM_length +
                                  rightThermalSwitch_length) /
                                 magnetizationSteps + 1)
                        second = (leftReservoir_length + (j + 1) *
                                  (leftThermalSwitch_length + MCM_length +
                                   rightThermalSwitch_length) /
                                  magnetizationSteps + 1)
                        a.activate(first, second)
                        totalmsteps = magnetizationSteps
                        delta_t = ((1 / (2 * (1. / (1. -
                                                    restingTimeHot -
                                                    restingTimeCold)) *
                                         freq *
                                         np.sqrt(totalmsteps))) *
                                   (np.sqrt(magnetizationSteps - j) -
                                    np.sqrt(magnetizationSteps - j -
                                            1)))
                        write_interval = int(1 / (freq * dt *
                                                  cyclePoints))
                        a.compute(delta_t, write_interval,
                                  solver=solverMode)

                # mode 6
                if magnetizationMode == 'decelerated_left':
                    for j in range(magnetizationSteps - 1, -1, -1):
                        first = (leftReservoir_length + j *
                                 (leftThermalSwitch_length + MCM_length +
                                  rightThermalSwitch_length) /
                                 magnetizationSteps + 1)
                        second = (leftReservoir_length + (j + 1) *
                                  (leftThermalSwitch_length + MCM_length +
                                   rightThermalSwitch_length) /
                                  magnetizationSteps + 1)
                        a.activate(first, second)
                        totalmsteps = magnetizationSteps
                        delta_t = ((1 / (2 * (1. / (1. -
                                                    restingTimeHot -
                                                    restingTimeCold)) *
                                         freq *
                                         np.sqrt(totalmsteps))) *
                                   (np.sqrt(j + 1) -
                                    np.sqrt(j)))
                        write_interval = int(1 / (freq * dt *
                                                  cyclePoints))
                        a.compute(delta_t, write_interval,
                                  solver=solverMode)

                a.compute(restingTimeHot * period,
                          int(1 / (freq * dt * cyclePoints)),
                          solver=solverMode)

                # DEMAGNETIZATION AND COMPUTATION

                # mode 1
                if demagnetizationMode == 'constant_right':
                    for j in range(demagnetizationSteps):
                        first = (leftReservoir_length + j *
                                 ((leftThermalSwitch_length + MCM_length +
                                   rightThermalSwitch_length) /
                                  demagnetizationSteps) +
                                 1)
                        second = (leftReservoir_length + (j + 1) *
                                  ((leftThermalSwitch_length + MCM_length +
                                    rightThermalSwitch_length) /
                                   demagnetizationSteps) +
                                  1)
                        a.deactivate(first, second)
                        time_interval = (((1 - restingTimeHot -
                                           restingTimeCold) *
                                          period / 2.) /
                                         (demagnetizationSteps))
                        write_interval = int(1 / (freq * dt *
                                                  cyclePoints))
                        a.compute(time_interval, write_interval,
                                  solver=solverMode)

                # mode 2
                if demagnetizationMode == 'accelerated_right':
                    for j in range(demagnetizationSteps):
                        # if restingTimeCold != 0.:
                        first = (leftReservoir_length + j *
                                 ((leftThermalSwitch_length + MCM_length +
                                   rightThermalSwitch_length) /
                                  demagnetizationSteps) +
                                 1)
                        second = (leftReservoir_length + (j + 1) *
                                  ((leftThermalSwitch_length + MCM_length +
                                    rightThermalSwitch_length) /
                                   demagnetizationSteps) +
                                  1)
                        a.deactivate(first, second)
                        totalsteps = demagnetizationSteps
                        delta_t = ((1 / (2 * (1. / (1. -
                                                    restingTimeHot -
                                                    restingTimeCold)) *
                                         freq * np.sqrt(totalsteps))) *
                                   (np.sqrt(j + 1) - np.sqrt(j)))
                        write_interval = int(1 / (freq * dt *
                                                  cyclePoints))
                        a.compute(delta_t, write_interval,
                                  solver=solverMode)

                # mode 3
                if demagnetizationMode == 'decelerated_right':
                    for j in range(demagnetizationSteps):
                            first = (leftReservoir_length + j *
                                     ((leftThermalSwitch_length + MCM_length +
                                       rightThermalSwitch_length) /
                                      demagnetizationSteps) +
                                     1)
                            second = (leftReservoir_length + (j + 1) *
                                      ((leftThermalSwitch_length + MCM_length +
                                        rightThermalSwitch_length) /
                                       demagnetizationSteps) +
                                      1)
                            a.deactivate(first, second)
                            totalsteps = demagnetizationSteps
                            delta_t = ((1 / (2 * (1. / (1. -
                                                        restingTimeHot -
                                                        restingTimeCold)) *
                                             freq * np.sqrt(totalsteps))) *
                                       (np.sqrt(demagnetizationSteps - j) -
                                        np.sqrt(demagnetizationSteps - j -
                                                1)))
                            write_interval = int(1 / (freq * dt *
                                                      cyclePoints))
                            a.compute(delta_t, write_interval,
                                      solver=solverMode)

                # mode 4
                if demagnetizationMode == 'constant_left':
                    for j in range(demagnetizationSteps - 1, -1, -1):
                        first = (leftReservoir_length + j *
                                 ((leftThermalSwitch_length + MCM_length +
                                   rightThermalSwitch_length) /
                                  demagnetizationSteps) +
                                 1)
                        second = (leftReservoir_length + (j + 1) *
                                  ((leftThermalSwitch_length + MCM_length +
                                    rightThermalSwitch_length) /
                                   demagnetizationSteps) +
                                  1)
                        a.deactivate(first, second)
                        time_interval = (((1 - restingTimeHot -
                                           restingTimeCold) *
                                          period / 2.) /
                                         (demagnetizationSteps - 1))
                        write_interval = int(1 / (freq * dt *
                                                  cyclePoints))
                        a.compute(time_interval, write_interval,
                                  solver=solverMode)

                # mode 5
                if demagnetizationMode == 'accelerated_left':
                    for j in range(demagnetizationSteps - 1, -1, -1):
                        first = (leftReservoir_length + j *
                                 ((leftThermalSwitch_length + MCM_length +
                                   rightThermalSwitch_length) /
                                  demagnetizationSteps) +
                                 1)
                        second = (leftReservoir_length + (j + 1) *
                                  ((leftThermalSwitch_length + MCM_length +
                                    rightThermalSwitch_length) /
                                   demagnetizationSteps) +
                                  1)
                        a.deactivate(first, second)
                        totalsteps = demagnetizationSteps
                        delta_t = ((1 / (2 * (1. / (1. -
                                                    restingTimeHot -
                                                    restingTimeCold)) *
                                         freq * np.sqrt(totalsteps))) *
                                   (np.sqrt(demagnetizationSteps - j) -
                                    np.sqrt(demagnetizationSteps - j -
                                            1)))
                        write_interval = int(1 / (freq * dt *
                                                  cyclePoints))
                        a.compute(delta_t, write_interval,
                                  solver=solverMode)

                # mode 6
                if demagnetizationMode == 'decelerated_left':
                    for j in range(demagnetizationSteps - 1, -1, -1):
                        first = (leftReservoir_length + j *
                                 ((leftThermalSwitch_length + MCM_length +
                                   rightThermalSwitch_length) /
                                  demagnetizationSteps) +
                                 1)
                        second = (leftReservoir_length + (j + 1) *
                                  ((leftThermalSwitch_length + MCM_length +
                                    rightThermalSwitch_length) /
                                   demagnetizationSteps) +
                                  1)
                        a.deactivate(first, second)
                        totalsteps = demagnetizationSteps
                        delta_t = ((1 / (2 * (1. / (1. -
                                                    restingTimeHot -
                                                    restingTimeCold)) *
                                         freq * np.sqrt(totalsteps))) *
                                   (np.sqrt(j + 1) - np.sqrt(j)))
                        write_interval = int(1 / (freq * dt *
                                                  cyclePoints))
                        a.compute(delta_t, write_interval,
                                  solver=solverMode)

                time_interval = restingTimeCold * period
                write_interval = int(1 / (freq * dt * cyclePoints))
                a.compute(time_interval, write_interval, solver=solverMode)

            if startingField == 'demagnetization':

                # DEMAGNETIZATION AND COMPUTATION

                # mode 1
                if demagnetizationMode == 'constant_right':
                    for j in range(demagnetizationSteps):
                        first = (leftReservoir_length + j *
                                 ((leftThermalSwitch_length + MCM_length +
                                   rightThermalSwitch_length) /
                                  demagnetizationSteps) +
                                 1)
                        second = (leftReservoir_length + (j + 1) *
                                  ((leftThermalSwitch_length + MCM_length +
                                    rightThermalSwitch_length) /
                                   demagnetizationSteps) +
                                  1)
                        a.deactivate(first, second)
                        time_interval = (((1 - restingTimeHot -
                                           restingTimeCold) *
                                          period / 2.) /
                                         (demagnetizationSteps))
                        write_interval = int(1 / (freq * dt *
                                                  cyclePoints))
                        a.compute(time_interval, write_interval,
                                  solver=solverMode)

                # mode 2
                if demagnetizationMode == 'accelerated_right':
                    for j in range(demagnetizationSteps):
                        first = (leftReservoir_length + j *
                                 ((leftThermalSwitch_length + MCM_length +
                                   rightThermalSwitch_length) /
                                  demagnetizationSteps) +
                                 1)
                        second = (leftReservoir_length + (j + 1) *
                                  ((leftThermalSwitch_length + MCM_length +
                                    rightThermalSwitch_length) /
                                   demagnetizationSteps) +
                                  1)
                        a.deactivate(first, second)
                        totalsteps = demagnetizationSteps
                        delta_t = ((1 / (2 * (1. / (1. -
                                                    restingTimeHot -
                                                    restingTimeCold)) *
                                         freq * np.sqrt(totalsteps))) *
                                   (np.sqrt(j + 1) - np.sqrt(j)))
                        write_interval = int(1 / (freq * dt *
                                                  cyclePoints))
                        a.compute(delta_t, write_interval,
                                  solver=solverMode)

                # mode 3
                if demagnetizationMode == 'decelerated_right':
                    for j in range(demagnetizationSteps):
                            first = (leftReservoir_length + j *
                                     ((leftThermalSwitch_length + MCM_length +
                                       rightThermalSwitch_length) /
                                      demagnetizationSteps) +
                                     1)
                            second = (leftReservoir_length + (j + 1) *
                                      ((leftThermalSwitch_length + MCM_length +
                                        rightThermalSwitch_length) /
                                       demagnetizationSteps) +
                                      1)
                            a.deactivate(first, second)
                            totalsteps = demagnetizationSteps
                            delta_t = ((1 / (2 * (1. / (1. -
                                                        restingTimeHot -
                                                        restingTimeCold)) *
                                             freq * np.sqrt(totalsteps))) *
                                       (np.sqrt(demagnetizationSteps - j) -
                                        np.sqrt(demagnetizationSteps - j -
                                                1)))
                            write_interval = int(1 / (freq * dt *
                                                      cyclePoints))
                            a.compute(delta_t, write_interval,
                                      solver=solverMode)

                # mode 4
                if demagnetizationMode == 'constant_left':
                    for j in range(demagnetizationSteps - 1, -1, -1):
                        first = (leftReservoir_length + j *
                                 ((leftThermalSwitch_length + MCM_length +
                                   rightThermalSwitch_length) /
                                  demagnetizationSteps) +
                                 1)
                        second = (leftReservoir_length + (j + 1) *
                                  ((leftThermalSwitch_length + MCM_length +
                                    rightThermalSwitch_length) /
                                   demagnetizationSteps) +
                                  1)
                        a.deactivate(first, second)
                        time_interval = (((1 - restingTimeHot -
                                           restingTimeCold) *
                                          period / 2.) /
                                         (demagnetizationSteps))
                        write_interval = int(1 / (freq * dt *
                                                  cyclePoints))
                        a.compute(time_interval, write_interval,
                                  solver=solverMode)

                # mode 5
                if demagnetizationMode == 'accelerated_left':
                    for j in range(demagnetizationSteps - 1, -1, -1):
                        first = (leftReservoir_length + j *
                                 ((leftThermalSwitch_length + MCM_length +
                                   rightThermalSwitch_length) /
                                  demagnetizationSteps) +
                                 1)
                        second = (leftReservoir_length + (j + 1) *
                                  ((leftThermalSwitch_length + MCM_length +
                                    rightThermalSwitch_length) /
                                   demagnetizationSteps) +
                                  1)
                        a.deactivate(first, second)
                        totalsteps = demagnetizationSteps
                        delta_t = ((1 / (2 * (1. / (1. -
                                                    restingTimeHot -
                                                    restingTimeCold)) *
                                         freq * np.sqrt(totalsteps))) *
                                   (np.sqrt(demagnetizationSteps - j) -
                                    np.sqrt(demagnetizationSteps - j -
                                            1)))
                        write_interval = int(1 / (freq * dt *
                                                  cyclePoints))
                        a.compute(delta_t, write_interval,
                                  solver=solverMode)

                # mode 6
                if demagnetizationMode == 'decelerated_left':
                    for j in range(demagnetizationSteps - 1, -1, -1):
                        first = (leftReservoir_length + j *
                                 ((leftThermalSwitch_length + MCM_length +
                                   rightThermalSwitch_length) /
                                  demagnetizationSteps) +
                                 1)
                        second = (leftReservoir_length + (j + 1) *
                                  ((leftThermalSwitch_length + MCM_length +
                                    rightThermalSwitch_length) /
                                   demagnetizationSteps) +
                                  1)
                        a.deactivate(first, second)
                        totalsteps = demagnetizationSteps
                        delta_t = ((1 / (2 * (1. / (1. -
                                                    restingTimeHot -
                                                    restingTimeCold)) *
                                         freq * np.sqrt(totalsteps))) *
                                   (np.sqrt(j + 1) - np.sqrt(j)))
                        write_interval = int(1 / (freq * dt *
                                                  cyclePoints))
                        a.compute(delta_t, write_interval,
                                  solver=solverMode)

                time_interval = restingTimeCold * period
                write_interval = int(1 / (freq * dt * cyclePoints))
                a.compute(time_interval, write_interval, solver=solverMode)

            # MAGNETIZATION AND COMPUTATION

                # mode 1
                if magnetizationMode == 'constant_right':
                    for j in range(magnetizationSteps):
                        first = (leftReservoir_length + j *
                                 (leftThermalSwitch_length + MCM_length +
                                  rightThermalSwitch_length) /
                                 magnetizationSteps + 1)
                        second = (leftReservoir_length + (j + 1) *
                                  (leftThermalSwitch_length + MCM_length +
                                   rightThermalSwitch_length) /
                                  magnetizationSteps + 1)
                        a.activate(first, second)
                        time_interval = ((1 - restingTimeHot -
                                          restingTimeCold) *
                                         period / 2.)
                        time_interval = (time_interval /
                                         (magnetizationSteps))
                        a.compute(time_interval,
                                  int(1 / (freq * dt * cyclePoints)),
                                  solver=solverMode)

                # mode 2
                if magnetizationMode == 'accelerated_right':
                    for j in range(magnetizationSteps):
                        first = (leftReservoir_length + j *
                                 (leftThermalSwitch_length + MCM_length +
                                  rightThermalSwitch_length) /
                                 magnetizationSteps + 1)
                        second = (leftReservoir_length + (j + 1) *
                                  (leftThermalSwitch_length + MCM_length +
                                   rightThermalSwitch_length) /
                                  magnetizationSteps + 1)
                        a.activate(first, second)
                        totalmsteps = magnetizationSteps
                        delta_t = ((1 / (2 * (1. / (1. -
                                                    restingTimeHot -
                                                    restingTimeCold)) *
                                         freq *
                                         np.sqrt(totalmsteps))) *
                                   (np.sqrt(j + 1) -
                                    np.sqrt(j)))
                        write_interval = int(1 / (freq * dt *
                                                  cyclePoints))
                        a.compute(delta_t, write_interval,
                                  solver=solverMode)

                # mode 3
                if magnetizationMode == 'decelerated_right':
                    for j in range(magnetizationSteps):
                        first = (leftReservoir_length + j *
                                 (leftThermalSwitch_length + MCM_length +
                                  rightThermalSwitch_length) /
                                 magnetizationSteps + 1)
                        second = (leftReservoir_length + (j + 1) *
                                  (leftThermalSwitch_length + MCM_length +
                                   rightThermalSwitch_length) /
                                  magnetizationSteps + 1)
                        a.activate(first, second)
                        totalmsteps = magnetizationSteps
                        delta_t = ((1 / (2 * (1. / (1. -
                                                    restingTimeHot -
                                                    restingTimeCold)) *
                                         freq *
                                         np.sqrt(totalmsteps))) *
                                   (np.sqrt(magnetizationSteps - j) -
                                    np.sqrt(magnetizationSteps - j -
                                            1)))
                        write_interval = int(1 / (freq * dt *
                                                  cyclePoints))
                        a.compute(delta_t, write_interval,
                                  solver=solverMode)

                # mode 4
                if magnetizationMode == 'constant_left':
                    for j in range(magnetizationSteps - 1, -1, -1):
                        first = (leftReservoir_length + j *
                                 (leftThermalSwitch_length + MCM_length +
                                  rightThermalSwitch_length) /
                                 magnetizationSteps + 1)
                        second = (leftReservoir_length + (j + 1) *
                                  (leftThermalSwitch_length + MCM_length +
                                   rightThermalSwitch_length) /
                                  magnetizationSteps + 1)
                        a.activate(first, second)
                        time_interval = (((1 - restingTimeHot -
                                          restingTimeCold) * period /
                                         2.) / (magnetizationSteps))
                        write_interval = int(1 / (freq * dt *
                                                  cyclePoints))
                        a.compute(time_interval, write_interval,
                                  solver=solverMode)

                # mode 5
                if magnetizationMode == 'accelerated_left':
                    for j in range(magnetizationSteps - 1, -1, -1):
                        first = (leftReservoir_length + j *
                                 (leftThermalSwitch_length + MCM_length +
                                  rightThermalSwitch_length) /
                                 magnetizationSteps + 1)
                        second = (leftReservoir_length + (j + 1) *
                                  (leftThermalSwitch_length + MCM_length +
                                   rightThermalSwitch_length) /
                                  magnetizationSteps + 1)
                        a.activate(first, second)
                        totalmsteps = magnetizationSteps
                        delta_t = ((1 / (2 * (1. / (1. -
                                                    restingTimeHot -
                                                    restingTimeCold)) *
                                         freq *
                                         np.sqrt(totalmsteps))) *
                                   (np.sqrt(magnetizationSteps - j) -
                                    np.sqrt(magnetizationSteps - j -
                                            1)))
                        write_interval = int(1 / (freq * dt *
                                                  cyclePoints))
                        a.compute(delta_t, write_interval,
                                  solver=solverMode)

                # mode 6
                if magnetizationMode == 'decelerated_left':
                    for j in range(magnetizationSteps - 1, -1, -1):
                        first = (leftReservoir_length + j *
                                 (leftThermalSwitch_length + MCM_length +
                                  rightThermalSwitch_length) /
                                 magnetizationSteps + 1)
                        second = (leftReservoir_length + (j + 1) *
                                  (leftThermalSwitch_length + MCM_length +
                                   rightThermalSwitch_length) /
                                  magnetizationSteps + 1)
                        a.activate(first, second)
                        totalmsteps = magnetizationSteps
                        delta_t = ((1 / (2 * (1. / (1. -
                                                    restingTimeHot -
                                                    restingTimeCold)) *
                                         freq *
                                         np.sqrt(totalmsteps))) *
                                   (np.sqrt(j + 1) -
                                    np.sqrt(j)))
                        write_interval = int(1 / (freq * dt *
                                                  cyclePoints))
                        a.compute(delta_t, write_interval,
                                  solver=solverMode)

                a.compute(restingTimeHot * period,
                          int(1 / (freq * dt * cyclePoints)),
                          solver=solverMode)

            if mode == 'heat_pump':
                if -(a.heatLeft - heatLeft) / period > maximumPower:
                    maximumPower = -(a.heatLeft - heatLeft) / period

                working_power = -((a.heatLeft - heatLeft) -
                                  (a.heatRight - heatRight)) / period
                if working_power > maximumWorkingPower:
                    maximumWorkingPower = - \
                        ((a.heatLeft - heatLeft) -
                         (a.heatRight - heatRight)) / period

            if mode == 'refrigerator':
                if -(a.heatRight - heatRight) / period > maximumPower:
                    maximumPower = -(a.heatRight - heatRight) / period

                working_power = -((a.heatLeft - heatLeft) -
                                  (a.heatRight - heatRight)) / period
                if working_power > maximumWorkingPower:
                    maximumWorkingPower = - \
                        ((a.heatLeft - heatLeft) -
                         (a.heatRight - heatRight)) / period

            if mode == 'refrigerator-energetics':
                coolingPower = -(a.heatRight - heatRight) / period
                workingPower = -((a.heatLeft - heatLeft) -
                                 (a.heatRight - heatRight)) / period
                COP = coolingPower / workingPower
                print (a.heatLeft - heatLeft), (a.heatRight - heatRight)

            if mode == 'heat_pump-energetics':
                heatingPower = -(a.heatLeft - heatLeft) / period
                workingPower = -((a.heatLeft - heatLeft) -
                                 (a.heatRight - heatRight)) / period
                COP = heatingPower / workingPower

            # updates the error values and prints information in log file
            value1 = value2
            value2 = a.temperature[rightTemperatureSensor][1]
            i = 1 + i

        # calculates the final temperatures
        before_final_cycle_time = a.timePassed - 1. / freq
        time_accumulated = 0.
        fileTemperature = open(fileName)
        lines = fileTemperature.readlines()
        final_temperature_right = 0.
        o = 1
        while float(lines[o].split(',')[0]) < before_final_cycle_time:
            lines.pop(0)
            o = o + 1
        for o in range(1, len(lines)):
            time_before = float(lines[o - 1].split(',')[0])
            time_now = float(lines[o].split(',')[0])
            sensor = rightTemperatureSensor - 2
            final_temperature_right = ((time_now - time_before) *
                                       float(lines[o].split(',')[sensor]) +
                                       final_temperature_right)
            time_accumulated = time_accumulated + (time_now - time_before)
        final_temperature_right = final_temperature_right / time_accumulated
        fileTemperature.close()

        before_final_cycle_time = a.timePassed - 1. / freq
        time_accumulated = 0.
        fileTemperature = open(fileName)
        lines = fileTemperature.readlines()
        final_temperature_left = 0.
        o = 1
        while float(lines[o].split(',')[0]) < before_final_cycle_time:
            lines.pop(0)
            o = o + 1
        for o in range(1, len(lines)):
            time_before = float(lines[o - 1].split(',')[0])
            time_now = float(lines[o].split(',')[0])
            sensor = leftTemperatureSensor + 1
            final_temperature_left = ((time_now - time_before) *
                                      float(lines[o].split(',')[sensor]) +
                                      final_temperature_left)
            time_accumulated = time_accumulated + (time_now - time_before)
        final_temperature_left = final_temperature_left / time_accumulated
        fileTemperature.close()

        # prints more information in log file
        endTime = time.time()
        simulationTime = endTime - startTime
        hours = int(simulationTime / 3600)
        minutes = int((simulationTime - hours * 3600) / 60)
        seconds = int(simulationTime - hours * 3600 - (minutes * 60))
        hours = '%02d' % hours
        minutes = '%02d' % minutes
        seconds = '%02d' % seconds

        print '------------------------------------------------------'
        print ''
        print 'Number of cycles:', i
        print 'Final cycle error:', abs((value1 - value2) / value2)
        if mode == 'refrigerator':
            print 'Maximum cooling power (W):', maximumPower
            print 'Maximum working power (W)', maximumWorkingPower
            temperature_span = (-final_temperature_right +
                                final_temperature_left)
            print 'No load temperature span (K):', temperature_span
        if mode == 'heat_pump':
            print 'Maximum heating power (W):', maximumPower
            print 'Maximum working power (W)', maximumWorkingPower
            temperature_span = final_temperature_right - final_temperature_left
            print 'No load temperature span (K):', temperature_span
        if mode == 'refrigerator-energetics':
            print 'Cooling power (W):', coolingPower
            print 'Working power (W):', workingPower
            print 'COP:', COP
        if mode == 'heat_pump-energetics':
            print 'Heating power (W):', heatingPower
            print 'Working power (W):', workingPower
            print 'COP:', COP
        print 'Final time (s):', a.timePassed
        print 'Simulation duration:', hours + ':' + minutes + ':' + seconds
        print ''
        print '------------------------------------------------------'
        print ''
        print ''
        print ''



# -*- coding: utf-8 -*-
from __future__ import unicode_literals
"""Contains the class heatcond_activemat_1D.

Used to compute a 1-dimensional model of heat conduction

"""

from numpy import *
import calmatpro
import os


class heatcond_activemat_1D:

    """heatcond_activemat_1D class

    This class solves numerically the heat conduction equation for 1 dimension
    of an active material(s). Three solvers can be used: explicit with
    x-independent k, explicit with x-dependent k, implicit with x-independent
    k, and implicit with x-dependent k. The class has 5 methods: activate for
    the activation of part of the solid, deactivate for the deactivation of
    part of the solid, and compute for solving the equation for a given period
    of time. This class is suited for simulations involving caloric systems
    such as magnetocaloric or electrocaloric systems.

    """

    def __init__(self, ambTemperature, materials=['Cu'], borders=[1, 11],
                 materialsOrder=[0], dx=0.01, dt=0.1, fileName='data.txt',
                 boundaries=[0, 0], Q=[], Q0=[], heatPoints=[1, -2],
                 initialState=False):
        """Initializes the object.

        ambTemperature: ambient temperature of the whole system
        materials: list of strings of all the used materials present in the
            folder materials
        borders: list of the points where there is a change of material
        materialsOrder: list of the materials list indexes that defines the
            material properties given by borders
        dx: the space step
        dt: the times step
        fileName: file name where the temperature and heat flux are saved
        boundaries: list of two entries that define the boundary condition
            for tempreture. If 0 the boundary condition is insulation
        Q: list of 3 entry lists that gives the fixed heat source coeficient.
            The first term is the initial space index where it is applies. The
            second is the final space index where it is applies. The third is
            the value of the coeficient.
        Q0 is a list of 3 entry lists that gives the temperature dependent heat
            source coefficient. The first term is the initial space index where
            it is applies. The second is the final space index where it is
            applies. The third is the value of the coeficient.
        heatPoints: list of the space indexes where we want to extract the heat
            flux. Normally, the first term is the heat flux of the hot end and
            the second term is the heat flux of the cold end

        """

        # initial definitions
        self.heatPoints = heatPoints
        self.borders = borders
        self.materials = range(len(materials))
        self.boundaries = boundaries
        self.ambTemperature = ambTemperature

        # loads the data for each material
        for i in range(len(materials)):
            tadi = os.path.dirname(os.path.realpath(__file__)) + \
                '/materials/' + materials[i] + '/' + 'tadi.txt'
            tadd = os.path.dirname(os.path.realpath(__file__)) + \
                '/materials/' + materials[i] + '/' + 'tadd.txt'
            cpa = os.path.dirname(os.path.realpath(__file__)) + \
                '/materials/' + materials[i] + '/' + 'cpa.txt'
            cp0 = os.path.dirname(os.path.realpath(__file__)) + \
                '/materials/' + materials[i] + '/' + 'cp0.txt'
            k0 = os.path.dirname(os.path.realpath(__file__)) + \
                '/materials/' + materials[i] + '/' + 'k0.txt'
            ka = os.path.dirname(os.path.realpath(__file__)) + \
                '/materials/' + materials[i] + '/' + 'ka.txt'
            rho0 = os.path.dirname(os.path.realpath(__file__)) + \
                '/materials/' + materials[i] + '/' + 'rho0.txt'
            rhoa = os.path.dirname(os.path.realpath(__file__)) + \
                '/materials/' + materials[i] + '/' + 'rhoa.txt'
            self.materials[i] = calmatpro.calmatpro(
                tadi, tadd, cpa, cp0, k0, ka, rho0, rhoa)

        # defines which are the properties of each material point
        self.materialsIndex = [None]
        for i in range(len(borders) - 1):
            for j in range(borders[i], borders[i + 1]):
                self.materialsIndex.append(materialsOrder[i])
        self.materialsIndex.append(None)

        # loads the inputs and creates the lists temperature, Cp, rho and k
        self.numPoints = borders[-1] + 1
        self.fileName = fileName
        self.dx = dx
        self.dt = dt
        self.temperature = [[ambTemperature, ambTemperature]]
        self.Cp = [None]
        self.rho = [None]
        self.Q = [None]
        self.Q0 = [None]
        self.k = [self.materials[self.materialsIndex[1]].k0(ambTemperature)]
        for i in range(1, borders[-1]):
            self.temperature.append([ambTemperature, ambTemperature])
            self.rho.append(self.materials[self.materialsIndex[i]].rho0(
                ambTemperature))  # rhoTSHot)
            self.Cp.append(
                self.materials[self.materialsIndex[i]].cp0(ambTemperature))
            self.k.append(
                self.materials[self.materialsIndex[i]].k0(ambTemperature))
            self.Q.append(0.)
            self.Q0.append(0.)
        self.temperature.append([ambTemperature, ambTemperature])
        self.rho.append(None)
        self.Cp.append(None)
        self.Q.append(None)
        self.Q0.append(None)
        self.k.append(
            self.materials[self.materialsIndex[-2]].k0(ambTemperature))

        # creates the power sources
        for power in Q:
            for j in range(power[1], power[2]):
                self.Q[j] = power[0]
        for power in Q0:
            for j in range(power[1], power[2]):
                self.Q0[j] = power[0]

        # initializes the state of each point (True if active and False if ...
        # unactive), the time, heat flux from the hot side and heat flux ...
        # from the cold side
        self.state = [initialState for i in range(borders[-1] + 1)]
        self.timePassed = 0.
        self.heatLeft = 0.
        self.heatRight = 0.

        line = 'time(s)'
        for i in range(len(self.temperature)):
            line = line + ',T[' + str(i) + '] (K)'
        line = line + ',heat[' + str(self.heatPoints[0]) + '](W)' + \
            ',heat[' + str(self.heatPoints[1]) + '](J)\n'
        f = open(self.fileName, 'a')
        f.write(line)
        f.close()

    def activate(self, initialPoint, finalPoint):
        """Activation of the material

        activates a given space interval of the material,
        between the initialPoint and finalPoint.

        """

        for i in range(initialPoint, finalPoint):

            if self.state[i] is False:
                self.temperature[i][0] = self.temperature[i][0] + \
                    self.materials[self.materialsIndex[i]].tadi(
                        self.temperature[i][0])
                self.rho[i] = self.materials[self.materialsIndex[i]].rhoa(
                    self.temperature[i][0])
                self.Cp[i] = self.materials[self.materialsIndex[i]].cpa(
                    self.temperature[i][0])
                self.k[i] = self.materials[self.materialsIndex[i]].ka(
                    self.temperature[i][0])
                self.state[i] = True

            else:
                message = 'point %f already activated' % float(i)
                print message

    def deactivate(self, initialPoint, finalPoint):
        """Deactivation of the material

        deactivates a given space interval of the material,
        between the initialPoint and finalPoint.

        """

        for i in range(initialPoint, finalPoint):

            if self.state[i] is True:
                self.temperature[i][0] = self.temperature[i][0] - \
                    self.materials[self.materialsIndex[i]].tadd(
                        self.temperature[i][0])
                self.rho[i] = self.materials[self.materialsIndex[i]].rho0(
                    self.temperature[i][0])
                self.Cp[i] = self.materials[self.materialsIndex[i]].cp0(
                    self.temperature[i][0])
                self.k[i] = self.materials[self.materialsIndex[i]].k0(
                    self.temperature[i][0])
                self.state[i] = False

            else:
                message = 'point %f already deactivated' % float(i)
                print message

    def changeHeatPower(self, Q=[], Q0=[]):
        """Heat power source change

        Changes the coeficients for the heat power sources

        """

        for power in Q:
            for j in range(power[1], power[2]):
                self.Q[j] = power[0]
        for power in Q0:
            for j in range(power[1], power[2]):
                self.Q0[j] = power[0]

    def compute(self, timeInterval, writeInterval, solver='explicit_k(x)',
                modeTemp=False, numFlag=0.5, modeTempPoint=1):
        """Computes the thermal process

        Computes the system for timeInterval, and writes into the fileName
        file every writeInterval time steps. Four different solvers can be
        used: 'explicit_general', 'explicit_k(x)', 'implicit_general',
        and 'implicit_k(x)'. heatPoints is a list that defines the points where
        the heat flux are calculated if modeTemp is True the compute method
        stops when the point modeTempPoint changes numFlag relative to the
        initial value

        """

        # number of time steps for the given timeInterval
        nt = int(timeInterval / self.dt)

        # number of time steps counting from the last writing process
        nw = 0

        # saves the reference temperature for the modeTemp
        initModeTemp = self.temperature[modeTempPoint][0]

        # computes
        for j in range(nt):

            # updates the timePassed
            self.timePassed = self.timePassed + self.dt

            # defines the material properties accoring to the state list
            for i in range(1, self.numPoints - 1):
                if self.state[i] is True:
                    self.rho[i] = self.materials[self.materialsIndex[i]].rhoa(
                        self.temperature[i][0])
                    self.Cp[i] = self.materials[self.materialsIndex[i]].cpa(
                        self.temperature[i][0])
                    self.k[i] = self.materials[self.materialsIndex[i]].ka(
                        self.temperature[i][0])
                if self.state[i] is False:
                    self.rho[i] = self.materials[self.materialsIndex[i]].rho0(
                        self.temperature[i][0])
                    self.Cp[i] = self.materials[self.materialsIndex[i]].cp0(
                        self.temperature[i][0])
                    self.k[i] = self.materials[self.materialsIndex[i]].k0(
                        self.temperature[i][0])

            # SOLVERS

            # implicit k constant
            if solver == 'implicit_general':

                # initializes the matrixes for the equation systems
                a = zeros((self.numPoints, self.numPoints))
                b = zeros(self.numPoints)

                # left boundary
                a[0][0] = 1
                if self.boundaries[0] == 0:
                    b[0] = self.temperature[1][0]
                else:
                    b[0] = self.boundaries[0]

                # right boundary
                a[self.numPoints - 1][self.numPoints - 1] = 1
                if self.boundaries[1] == 0:
                    value = self.temperature[self.numPoints - 2][0]
                    b[self.numPoints - 1] = value
                else:
                    b[self.numPoints - 1] = self.boundaries[1]

                # creates the matrixes and solves the equation systems
                for i in range(1, self.numPoints - 1):
                    beta = self.k[i] * self.dt / \
                        (2 * self.rho[i] * self.Cp[i] * self.dx * self.dx)
                    sigma = self.dt / (self.rho[i] * self.Cp[i])

                    a[i][i - 1] = -beta
                    a[i][i] = 1 + 2 * beta - sigma * self.Q[i]
                    a[i][i + 1] = -beta
                    b[i] = (1 - 2 * beta - sigma * self.Q[i]) * \
                        self.temperature[i][0] + \
                        beta * self.temperature[i + 1][0] + \
                        beta * self.temperature[i - 1][0] + 2. * sigma * \
                        (self.Q0[i] - self.Q[i] * self.ambTemperature)

                x = linalg.solve(a, b)

                # updates the temperature list
                for i in range(self.numPoints):
                    self.temperature[i][1] = x[i]
                    self.temperature[i][0] = x[i]

            # implicit k dependent on x
            if solver == 'implicit_k(x)':

                # initializes the matrixes for the equation systems
                a = zeros((self.numPoints, self.numPoints))
                b = zeros(self.numPoints)

                # left boundary
                a[0][0] = 1
                if self.boundaries[0] == 0:
                    b[0] = self.temperature[1][0]
                else:
                    b[0] = self.boundaries[0]

                # right boundary
                a[self.numPoints - 1][self.numPoints - 1] = 1
                if self.boundaries[1] == 0:
                    value = self.temperature[self.numPoints - 2][0]
                    b[self.numPoints - 1] = value
                else:
                    b[self.numPoints - 1] = self.boundaries[1]

                # creates the matrixes and solves the equation systems
                for i in range(1, self.numPoints - 1):
                    gamma = 4. * self.rho[i] * self.Cp[i] * \
                        self.dx * self.dx / self.dt

                    a[i][i - 1] = self.k[i - 1] + self.k[i]
                    a[i][i] = -(gamma + self.k[i + 1] + self.k[i - 1] +
                                2. * self.k[i] - 2 * self.dt * self.dt *
                                self.Q[i])
                    a[i][i + 1] = self.k[i + 1] + self.k[i]
                    b[i] = -(self.k[i + 1] + self.k[i]) * \
                        self.temperature[i + 1][0] + \
                        (-gamma + self.k[i + 1] + self.k[i - 1] + 2. *
                            self.k[i] - 2 * self.dt * self.dt * self.Q[i]) * \
                        self.temperature[i][0] - \
                        (self.k[i - 1] + self.k[i]) * \
                        self.temperature[i - 1][0] - \
                        4. * self.dx * self.dx * \
                        (self.Q0[i] - self.Q[i] * self.ambTemperature)

                x = linalg.solve(a, b)

                # updates the temperature list
                for i in range(self.numPoints):
                    self.temperature[i][1] = x[i]
                    self.temperature[i][0] = x[i]

            # explicit k constant
            if solver == 'explicit_general':

                # computes
                for i in range(1, self.numPoints - 1):

                    alpha = self.dt * \
                        self.k[i] / (self.rho[i] * self.Cp[i] *
                                     self.dx * self.dx)
                    beta = self.dt / (self.rho[i] * self.Cp[i])

                    Tnew = (1 + beta * self.Q[i]) * self.temperature[i][0] + \
                        alpha * (self.temperature[i - 1][0] -
                                 2 * self.temperature[i][0] +
                                 self.temperature[i + 1][0]) + \
                        beta * (self.Q0[i] - self.Q[i] * self.ambTemperature)
                    self.temperature[i][1] = Tnew

                # left boundary for next time step
                if self.boundaries[0] == 0:
                    self.temperature[0][1] = self.temperature[1][0]
                else:
                    self.temperature[0][1] = self.boundaries[0]

                # right boundary for next time step
                if self.boundaries[1] == 0:
                    self.temperature[self.numPoints - 1][1] = \
                        self.temperature[self.numPoints - 2][0]
                else:
                    self.temperature[self.numPoints -
                                     1][1] = self.boundaries[1]

                # updates the temperature list
                for i in range(self.numPoints):
                    self.temperature[i][0] = self.temperature[i][1]

            # explicit k dependent on x
            if solver == 'explicit_k(x)':

                # computes
                for i in range(1, self.numPoints - 1):

                    eta = self.dt / \
                        (2. * self.rho[i] * self.Cp[i] * self.dx * self.dx)
                    beta = self.dt / (self.rho[i] * self.Cp[i])

                    Tnew = (1 + beta * self.Q[i]) * self.temperature[i][0] + \
                        eta * ((self.k[i + 1] + self.k[i]) *
                               self.temperature[i + 1][0] -
                               (self.k[i - 1] + self.k[i + 1] + 2 *
                                self.k[i]) *
                               self.temperature[i][0] +
                               (self.k[i - 1] + self.k[i]) *
                               self.temperature[i - 1][0]) + \
                        beta * (self.Q0[i] - self.Q[i] * self.ambTemperature)

                    self.temperature[i][1] = Tnew

                # left boundary for next time step
                if self.boundaries[0] == 0:
                    self.temperature[0][1] = self.temperature[1][1]
                else:
                    self.temperature[0][1] = self.boundaries[0]

                # right boundary for next time step
                if self.boundaries[1] == 0:
                    self.temperature[self.numPoints - 1][1] = \
                        self.temperature[self.numPoints - 2][1]
                else:
                    self.temperature[self.numPoints -
                                     1][1] = self.boundaries[1]

                # updates the temperature list
                for i in range(self.numPoints):
                    self.temperature[i][0] = self.temperature[i][1]

            # calculates the heat flux of the defined ...
            # two points during the time step
            self.heatLeft = -(self.k[self.heatPoints[0]] *
                              self.dt / self.dx) * \
                (self.temperature[self.heatPoints[0]][1] -
                 self.temperature[self.heatPoints[0] - 1][1]) + \
                self.heatLeft
            self.heatRight = -(self.k[self.heatPoints[1]] *
                               self.dt / self.dx) * \
                (self.temperature[self.heatPoints[1]][1] -
                 self.temperature[self.heatPoints[1] - 1][1]) + \
                self.heatRight

            nw = nw + 1

            # stops the simulation if modeTemp is True ...
            # and the stop condition is verified
            if modeTemp is True:
                value = self.temperature[modeTempPoint][0] - initModeTemp
                if abs(value) > numFlag:
                    nw = 0
                    f = open(self.fileName, 'a')
                    f.write(line)
                    f.close()
                    break

            # writes the temperature to fileName file ...
            # if the number of time steps is verified
            if nw == writeInterval or j == 0 or j == nt - 1:
                line = '%f,' % self.timePassed
                for i in self.temperature:
                    newLine = '%f,' % i[1]
                    line = line + newLine
                newLine = '%f,' % self.heatLeft
                line = line + newLine
                newLine = '%f' % self.heatRight
                line = line + newLine + '\n'
                f = open(self.fileName, 'a')
                f.write(line)
                f.close()

            if nw == writeInterval:
                nw = 0



# -*- coding: utf-8 -*-
from __future__ import unicode_literals
"""Heatrapy.

This module contains classes to extract the physical properties from
the materials from the heatrapy database, to create and compute
1 dimensional models of heat conduction, and to use the 1 dimensional
model to build models that include the magnetocaloric technology.

The package relies on 3 classes:

#########
calmatpro
#########

Extracts and interpolate the physical properties density (rho), specific
heat (cp), adiabatic change of temperature (tad), and thermal conductivity
(k), giving the temperature.

#########
heatcond
#########

Creates 1 dimensional models and computes the system using one of 4 solvers:
implicit with k(x), implicit with constant k, explicit with k(x), and explicit
with constant k. The output data is saved in a file.

#########
magcalsys
#########

Creates 1 dimensional models of magnetocaloric systems using the heatcond
class. During the computation, it writes log files of the key parameters that
can be analyzed at the end.

"""

from heatcond import heatcond_activemat_1D
from magcalsys import magcalsys_solidstate_1D

__all__ = [heatcond_activemat_1D, magcalsys_solidstate_1D]
